@@ -1,10 +1,8 @@
 /*
- * Copyright (c) 1998-2000 Apple Computer, Inc. All rights reserved.
+ * Copyright (c) 1998-2003 Apple Computer, Inc. All rights reserved.
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -473,11 +471,17 @@ IOMedia * IOApplePartitionScheme::instantiateMediaObject(
     IOMedia * media               = getProvider();
     UInt64    mediaBlockSize      = media->getPreferredBlockSize();
     UInt64    partitionBase       = 0;
-    char *    partitionHint       = partition->dpme_type;
+    char      partitionHint[DPISTRLEN + 1];
     bool      partitionIsWritable = media->isWritable();
-    char *    partitionName       = partition->dpme_name;
+    char      partitionName[DPISTRLEN + 1];
     UInt64    partitionSize       = 0;
 
+    strncpy(partitionHint, partition->dpme_type, DPISTRLEN);
+    strncpy(partitionName, partition->dpme_name, DPISTRLEN);
+
+    partitionHint[DPISTRLEN] = 0;
+    partitionName[DPISTRLEN] = 0;
+
     // Compute the relative byte position and size of the new partition.
 
     partitionBase  = OSSwapBigToHostInt32(partition->dpme_pblock_start);
@@ -504,15 +508,12 @@ IOMedia * IOApplePartitionScheme::instantiateMediaObject(
                        /* type     */ OSString,
                        /* instance */ hintTable->getObject(partitionHint) );
 
-        if ( hintValue ) partitionHint = (char *) hintValue->getCStringNoCopy();
+        if ( hintValue )
+        {
+            strncmp(partitionHint, hintValue->getCStringNoCopy(), DPISTRLEN);
+        }
     }
 
-    // Look up a name for the new partition.
-
-    while ( *partitionName == ' ' )  { partitionName++; }
-
-    if ( *partitionName == 0 )  partitionName = 0;
-
     // Determine whether the new partition type is Apple_Free, which we choose
     // not to publish because it is an internal concept to the partition map.
 
@@ -554,7 +555,7 @@ IOMedia * IOApplePartitionScheme::instantiateMediaObject(
 
             char name[24];
             sprintf(name, "Untitled %ld", partitionID);
-            newMedia->setName(partitionName ? partitionName : name);
+            newMedia->setName(partitionName[0] ? partitionName : name);
 
             // Set a location value (the partition number) for this partition.
 
@@ -598,67 +599,33 @@ bool IOApplePartitionScheme::attachMediaObjectToDeviceTree( IOMedia * media )
     // Attach the given media object to the device tree plane.
     //
 
-    const char * name = 0;
-    IOService *  service;
-    SInt32       unit = -1;
+    IORegistryEntry * parent = this;
 
-    for ( service = this; service; service = service->getProvider() )
+    while ( (parent = parent->getParentEntry(gIOServicePlane)) )
     {
-        OSNumber * number;
-        OSString * string;
-
-        if ( (number = OSDynamicCast(OSNumber, service->getProperty("IOUnit"))))
+        if ( parent->inPlane(gIODTPlane) )
         {
-            if ( unit == -1 )
-            {
-                unit = number->unsigned32BitValue();
-            }
-        }
+            char         location[ 32 ];
+            const char * locationOfParent = parent->getLocation(gIODTPlane);
+            const char * nameOfParent     = parent->getName(gIODTPlane);
 
-        if ( (string = OSDynamicCast(OSString, service->getProperty("IOUnitName"))))
-        {
-            if ( name == 0 )
-            {
-                name = string->getCStringNoCopy();
-            }
-        }
+            if ( locationOfParent == 0 )  break;
 
-        if ( service->inPlane(gIODTPlane) )
-        {
-            IORegistryEntry *    child;
-            IORegistryIterator * children;
+            if ( OSDynamicCast(IOMedia, parent) == 0 )  break;
 
-            if ( unit == -1 || service->getProvider() == 0 )  break;
+            parent = parent->getParentEntry(gIODTPlane);
 
-            children = IORegistryIterator::iterateOver(service, gIODTPlane);
+            if ( parent == 0 )  break;
 
-            if ( children == 0 )  break;
+            if ( media->attachToParent(parent, gIODTPlane) == false )  break;
 
-            while ( (child = children->getNextObject()) )
-            {
-                if ( OSDynamicCast(IOMedia, child) == 0 )
-                {
-                    child->detachAll(gIODTPlane);
-                }
-            }
+            strcpy(location, locationOfParent);
+            if ( strchr(location, ':') )  *(strchr(location, ':') + 1) = 0;
+            strcat(location, media->getLocation());
+            media->setLocation(location, gIODTPlane);
+            media->setName(nameOfParent, gIODTPlane);
 
-            children->release();
-
-            if ( media->attachToParent(service, gIODTPlane) )
-            {
-                char location[ sizeof("hhhhhhhh:dddddddddd") ];
-
-                if ( name == 0 )  name = "";
-
-                sprintf(location, "%lx:", unit);
-                strcat(location, media->getLocation());
-                media->setLocation(location, gIODTPlane);
-                media->setName(name, gIODTPlane);
-
-                return true;
-            }
-
-            break;
+            return true;
         }
     }
 
